@@ -2,7 +2,7 @@
 import inspect
 from dataclasses import Field, dataclass
 from typing import (Any, Awaitable, Callable, Mapping, MutableMapping,
-                    Optional, Sequence, Tuple, Type, Union, cast)
+                    NamedTuple, Optional, Sequence, Tuple, Type, Union, cast)
 
 service = dataclass
 
@@ -13,6 +13,23 @@ AsyncCleanup = Callable[[Any], Awaitable[None]]
 Cleanup = Union[SyncCleanup, AsyncCleanup]
 
 
+class ContainerEntry(NamedTuple):
+    """Configuration for a container entry."""
+
+    factory: Factory
+    cleanup: Optional[Cleanup]
+    cache: bool
+
+    async def run_cleanup(self, instance: Any):
+        """Run the cleanup function on an instance of the service."""
+        if not self.cleanup:
+            return
+
+        result = self.cleanup(instance)
+        if inspect.isawaitable(result):
+            await cast(Awaitable[Any], result)
+
+
 class Container:
     """DependencyInjection container instance."""
 
@@ -25,9 +42,15 @@ class Container:
         return key in self.services
 
     def get(self, key: Key) -> Any:
+        """Get an instance of a class registered in the container."""
         key = _get_key(key)
+        entry = self.container_definition.get(key)
+
+        if not entry.cache:
+            return entry.factory(self)
+
         if key not in self.services:
-            self.services[key] = self.container_definition.get(key)(self)
+            self.services[key] = entry.factory(self)
         return self.services[key]
 
     async def reset(self, key: Key) -> "Container":
@@ -49,15 +72,8 @@ class Container:
         if key not in self.services:
             return
 
-        cleanup = self.container_definition.get_cleanup(key)
-
-        if not cleanup:
-            return
-
-        service = self.services[key]
-        result = cleanup(service)
-        if inspect.isawaitable(result):
-            await cast(Awaitable[Any], result)
+        instance = self.services[key]
+        await self.container_definition.get(key).run_cleanup(instance)
 
     async def cleanup_all(self):
         """Iterate through all services that have been created, and call the associated cleanup
@@ -73,11 +89,15 @@ class ContainerDefinition:
 
     def __init__(self, *, allow_overwrite: bool = False) -> None:
         self.allow_overwrite: bool = allow_overwrite
-        self.services: MutableMapping[str, Factory] = {}
-        self.cleanup: MutableMapping[str, Cleanup] = {}
+        self.services: MutableMapping[str, ContainerEntry] = {}
 
     def add_factory(
-        self, key: Key, factory: Factory, *, cleanup: Optional[Cleanup] = None
+        self,
+        key: Key,
+        factory: Factory,
+        *,
+        cleanup: Optional[Cleanup] = None,
+        cache: bool = True,
     ) -> "ContainerDefinition":
         """Register a service in the container with an explicit key and factory."""
         key = _get_key(key)
@@ -86,10 +106,7 @@ class ContainerDefinition:
             if key in self.services:
                 raise KeyError(f"Key {key} already added to container")
 
-        self.services[key] = factory
-
-        if cleanup:
-            self.cleanup[key] = cleanup
+        self.services[key] = ContainerEntry(factory, cleanup, cache)
 
         return self
 
@@ -100,6 +117,7 @@ class ContainerDefinition:
         factory: Optional[Factory] = None,
         *,
         cleanup: Optional[Cleanup] = None,
+        cache: bool = True,
     ) -> "ContainerDefinition":
         """Register a service in the container with an explicit key and a list of keys of
         dependencies.
@@ -115,17 +133,22 @@ class ContainerDefinition:
             key,
             _key_list_factory([_get_key(k) for k in dependencies], factory),
             cleanup=cleanup,
+            cache=cache,
         )
 
     def add_service(
-        self, cls: Type, *, cleanup: Optional[Cleanup] = None,
+        self,
+        cls: Type,
+        *,
+        cleanup: Optional[Cleanup] = None,
+        cache: bool = True,
     ) -> "ContainerDefinition":
         """Register a type wrapped in @service. All of the typed properties on the class will
         be used as the dependency list.
         """
         dependency_dict: Mapping[str, Field] = cls.__dataclass_fields__
         dependency_keys = [_get_key(field.type) for field in dependency_dict.values()]
-        return self.add_key_list(cls, dependency_keys, cleanup=cleanup)
+        return self.add_key_list(cls, dependency_keys, cleanup=cleanup, cache=cache)
 
     def add(
         self,
@@ -135,6 +158,7 @@ class ContainerDefinition:
         ] = None,
         *,
         cleanup: Optional[Cleanup] = None,
+        cache: bool = True,
     ) -> "ContainerDefinition":
         """Catch all method that will call either add_factory, add_key_list, or add_service
         depending on the arguments.
@@ -148,23 +172,22 @@ class ContainerDefinition:
                 raise TypeError(
                     "key must be a Type if dependencies and factory are not given"
                 )
-            return self.add_service(key, cleanup=cleanup)
+            return self.add_service(key, cleanup=cleanup, cache=cache)
         if callable(factory):
-            return self.add_factory(key, factory, cleanup=cleanup)
+            return self.add_factory(key, factory, cleanup=cleanup, cache=cache)
         if isinstance(factory, tuple):
             (dependencies, factory_fn) = factory
-            return self.add_key_list(key, dependencies, factory_fn, cleanup=cleanup)
-        return self.add_key_list(key, factory, cleanup=cleanup)
+            return self.add_key_list(
+                key, dependencies, factory_fn, cleanup=cleanup, cache=cache
+            )
+        return self.add_key_list(key, factory, cleanup=cleanup, cache=cache)
 
-    def get(self, key: str) -> Factory:
+    def get(self, key: str) -> ContainerEntry:
         return self.services[key]
 
     def get_container(self) -> Container:
         return Container(self)
 
-    def get_cleanup(self, key: str) -> Optional[Cleanup]:
-        return self.cleanup.get(key, None)
-
 
 def _key_list_factory(dependencies: Sequence[str], factory: Callable) -> Factory:
     def build(container: Container) -> Any:
